#
# landmark-proj-est, 28 Aug 21
#
# Data from:
# Schedule Estimation and Uncertainty Surrounding the Cone of Uncertainty
# Todd Little
#
# Example from:
# Evidence-based Software Engineering: based on the publicly available data
# Derek M. Jones
#
# TAG project_estimate project_actual project_percentage-complete


library("colorspace")
library("plyr")


pal_col=rainbow(3)

par(bty="l")
par(las=1)
par(pch="+")


# Project Code,Week End Date,Target RelDate,Env,Plan,Dev,Stab,Mob,Adj Week,Est EndDate,First Est,Actual Release,Start Date,X,Absolute Ratio,Relative Ratio,Update,Absolute ratio without factor 2,,Eind project,Oppervlakte,Counter,Som oppervlakte,EQF all,EQF end,Eind project factor 2,Oppervlakte factor 2,Counter factor 2,Som oppervlakte factor 2,EQF all factor 2,EQF end factor 2
# 1,12/01/00,03/01/01,C,C,I,,,12/01/00,03/01/01,03/01/01,08/22/01,12/01/00,0.00,0.6770156819,2.95,1,0.33851,,0,0.0085951791,1,0,4235.7284514877,0,0,0.0176034663,1,0,#DIV/0!,0
est=read.csv(paste0(ESEUR_dir, "projects/Little06.csv.xz"), as.is=TRUE)

est$Week.End.Date=as.Date(est$Week.End.Date, format="%m/%d/%y")
est$Target.RelDate=as.Date(est$Target.RelDate, format="%m/%d/%y")
est$Adj.Week=as.Date(est$Adj.Week, format="%m/%d/%y")
est$Est.EndDate=as.Date(est$Est.EndDate, format="%m/%d/%y")
est$First.Est=as.Date(est$First.Est, format="%m/%d/%y")
est$Actual.Release=as.Date(est$Actual.Release, format="%m/%d/%y")
est$Start.Date=as.Date(est$Start.Date, format="%m/%d/%y")
est$act_duration=est$Actual.Release-est$Start.Date
est$est_duration=est$First.Est-est$Start.Date

est$percent_comp=100*as.integer(est$Week.End.Date-est$Start.Date)/as.integer(est$Actual.Release-est$Start.Date)
est$AE_ratio=as.integer(est$Actual.Release-est$Start.Date)/as.integer(est$Target.RelDate-est$Start.Date)


mk_target_unique=function(df)
{
# The same Target release date might be estimated again after a change of date,
# so duplicated cannot be used.  rle only works with atomic types
t=cumsum(c(1, head(rle(as.integer(df$Target.RelDate))$lengths, n=-1)))
return(df[t, ])
}


est=subset(est, Week.End.Date <= Actual.Release)
u_est=ddply(est, .(Project.Code), mk_target_unique)

# There are three cases where this relation is not true, and it should to be.
u_est=subset(u_est, Week.End.Date <= Target.RelDate)

plot(u_est$percent_comp,u_est$AE_ratio, log="y", col=pal_col[2],
	xaxs="i",
	xlim=c(-1, 100),
	xlab="Percentage completed", ylab="Actual/Estimated")

x_vals=10:100
lines(x_vals, 100/x_vals, col=pal_col[1])

lines(c(0, 100), c(1, 1), col=pal_col[3])

dev.copy(dev=png, file="landmark_act-est.png")
dev.off()


pal_col=rainbow(4)

num_proj_estimates=function(df)
{
return(data.frame(est_duration=df$est_duration[1], act_duration=df$act_duration[1], num_reest=nrow(df)))
}


t_sum=ddply(u_est, .(Project.Code), num_proj_estimates)

plot(t_sum$est_duration, t_sum$num_reest, col=pal_col[2],
     	xlim=c(0, 650),
	xlab="Days", ylab="Estimates per project")

lines(loess.smooth(t_sum$est_duration, t_sum$num_reest, span=0.5, family="gaussian"), col=pal_col[1])

points(t_sum$act_duration, t_sum$num_reest, col=pal_col[4])

lines(loess.smooth(t_sum$act_duration, t_sum$num_reest, span=0.5, family="gaussian"), col=pal_col[3])

legend(x="topright", legend=c("loess fit (initial)", "Initial estimate", "loess fit (actual)", "Actual days"), bty="n", fill=pal_col, cex=1.2)


